#import "BexFlutterPlugin.h"
#import "BexSdk.h"

@implementation BexFlutterPlugin

#define SET_DEBUG_MODE @"setDebugMode"
#define SUBMIT_CONSUMER @"submitConsumer"
#define RESUBMIT_CONSUMER @"resubmitConsumer"
#define PAYMENT @"payment"

static BexFlutterPlugin* instance = nil;

+ (BexFlutterPlugin*) sharedPlugin {
    return instance;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
  @synchronized(self) {
    if (instance == nil) {
      instance = [[BexFlutterPlugin alloc] init:registrar];
      [registrar addApplicationDelegate:instance];
    }
  }
}

- (instancetype)init:(NSObject<FlutterPluginRegistrar> *)registrar {
    self = [super init];
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"bex_flutter_plugin" binaryMessenger:[registrar messenger]];
    self.callbackChannel = [FlutterMethodChannel methodChannelWithName:@"bex_flutter_plugin/callback" binaryMessenger:[registrar messenger]];
    [registrar addMethodCallDelegate:self channel:channel];
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([SET_DEBUG_MODE isEqualToString:call.method]) {
      NSNumber* isDebugMode = call.arguments[@"isDebugMode"];
      [BexSdk setDebugMode: [isDebugMode boolValue]];
  } else if ([SUBMIT_CONSUMER isEqualToString:call.method]) {
      NSString* token = call.arguments[@"token"];
      [BexSdk submitConsumer:token];
  } else if ([RESUBMIT_CONSUMER isEqualToString:call.method]) {
      NSString* ticket = call.arguments[@"ticket"];
      [BexSdk resubmitConsumer:ticket];
  } else if ([PAYMENT isEqualToString:call.method]) {
      NSString* token = call.arguments[@"token"];
      [BexSdk payment:token];
  } else {
    result(FlutterMethodNotImplemented);
  }
}

- (void)sendMessage:(NSString*)method params:(NSDictionary*)params {
    [self.callbackChannel invokeMethod:method arguments:params];
}

@end
